mod basic;
mod test_basic;

// mod test_zeros_are_not_written;

mod test_map;
mod test_map_pb;

// mod test_enum_alias_pb;
// mod test_enum_alias;

mod test_ident_pb;

mod test_oneof;
mod test_oneof_pb;
