//! A module to import many generated files plus some custom tests
//!
//! For most generated files, the 'test' is just making sure it compiles!

pub mod basic;
pub mod issue118;
pub mod issue118_2;
pub mod nonunique_1;
pub mod nonunique_2;
pub mod special;
pub mod struct_pb;
pub mod test_basic;
pub mod test_default_values;
pub mod test_default_values_test;
pub mod test_enum_values_pb;
pub mod test_ident_pb;
pub mod test_import_nested_imported_pb;
pub mod test_import_nested_pb;
pub mod test_import_nonunique_pb;
pub mod test_import_root_imported_pb;
pub mod test_import_root_pb;
pub mod test_lite_runtime;
pub mod test_lite_runtime_test;
pub mod test_nonunique_enum_pb;
pub mod test_oneof;
pub mod test_oneof_pb;
pub mod test_required;
pub mod test_required_test;
pub mod test_sanitize_file_name_pb;
pub mod test_sync;
pub mod test_text_format;
pub mod test_text_format_pb;
