syntax = "proto2";

message Vec { }

message String { }

message Option { }
message None { }
message Some { }

message Message { }

message TestType {
    oneof type {
        string s = 1;
    }
    repeated string struct = 2;
    repeated uint32 ref = 3;
}
