enum City {
    LONDON = 0;
    PARIS = 1;
}

message Address {
    City city = 1;
}

message Person {
    Address address = 2;
    repeated string names = 3;
}

message PersonPacked {
    Address address = 2;
    repeated string names = 3 [packed=true];
}
